/**
 * @file src/index.js
 * @license MIT
 * @copyright 2017-present Foko Inc. All rights reserved.
 */

const getPrototypeOf = Object.getPrototypeOf || function legacyGetPrototypeOf (o) {
  /* eslint no-proto: 0 */
  return o.__proto__
}

/**
 * Creates a dirty JSON signature for quick ETag verification.
 * @param {Object} json stringifiable object
 * @param {Number} [max=10] maximum depth to search
 * @param {Array} seen internally used to manage circular object
 */
function genSig (json, max = 10, seen = []) {
  // protect against circular objects
  if (seen.length >= max || seen.indexOf(json) !== -1 || json === undefined) return ''
  seen.push(json)

  const type = typeof json

  // quick exit for primitives
  if (type === 'function' || json === null || Number.isNaN(json)) return 'null'
  if (type === 'string') return `'${json.replace(/[^\x21\x23-\x7E\x80-\xFF]+/g, '')}'`

  // number & boolean
  if (type !== 'object') return json

  // built-ins
  const proto = getPrototypeOf(json)

  if (proto !== null) {
    if ('toLocaleTimeString' in proto) return +json
    if ('test' in proto && 'exec' in proto) return json
  }

  // quick stringify
  let sigg = 'length' in json && 'map' in json ? '[' : '{'
  for (const k in json) {
    sigg += k + ':' + genSig(json[k], max, seen)
  }
  return sigg
}

/**
 * Generates an ETag for a given entity.
 * @param {any} body entity for which to generate a tag
 * @returns a weak ETag representing the entity
 */
export default function createEtag (body, depth) {
  return `W/"${genSig(body, depth)}"`
}
